package gov.grants.apply.dao;

import gov.grants.apply.valueObject.GetOpportunityList;

import java.sql.*;

import java.util.List;
import java.util.Vector;

/**
 * @author Enoch Moses
 * 
 * GetOpportunity Data Access Object (DAO). This class contains all database
 * handling that is needed to permanently store and retrieve GetOpportunity
 * object instances.
 * 
 * Modified to work w
 */

public class GetOpportunityListDao {
	private int CaseNumber = 1;

	/**
	 * load-method. This will load valueObject contents from database using
	 * Primary-Key as identifier. Upper layer should use this so that
	 * valueObject instance is created and only primary-key should be specified.
	 * Then call this method to complete other persistent information. This
	 * method will overwrite all other fields except primary-key and possible
	 * runtime variables. If load can not find matching row, NotFoundException
	 * will be thrown.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 * @param valueObject
	 *            This parameter contains the class instance to be loaded.
	 *            Primary-key field must be set for this to work properly.
	 */
	public void load(Connection conn, GetOpportunityList valueObject)
			throws SQLException {

		String sql = "SELECT * FROM getopportunity WHERE";

		//==========================================================
		//  Start of the Ending of the Dynamic "Where" Statement
		//==========================================================
		if (((valueObject.getOPPORTUNITY_ID() != null) || (valueObject
				.getOPPORTUNITY_ID().length() > 0))
				&& ((valueObject.getCFDA_NUMBER() == null) || (valueObject
						.getCFDA_NUMBER().length() == 0))
				&& ((valueObject.getCOMPETITION_ID() == null) || (valueObject
						.getCOMPETITION_ID().length() == 0))) {
			sql = sql + " (OPPORTUNITY_ID = ?)";
		} else if (((valueObject.getOPPORTUNITY_ID() != null) || (valueObject
				.getOPPORTUNITY_ID().length() > 0))
				&& ((valueObject.getCFDA_NUMBER() != null) || (valueObject
						.getCFDA_NUMBER().length() > 0))
				&& ((valueObject.getCOMPETITION_ID() == null) || (valueObject
						.getCOMPETITION_ID().length() == 0))) {
			sql = sql + "(OPPORTUNITY_ID = ? AND CFDA_NUMBER = ?)";
			CaseNumber = 2;
		} else if (((valueObject.getOPPORTUNITY_ID() != null) || (valueObject
				.getOPPORTUNITY_ID().length() > 0))
				&& ((valueObject.getCFDA_NUMBER() == null) || (valueObject
						.getCFDA_NUMBER().length() == 0))
				&& ((valueObject.getCOMPETITION_ID() != null) || (valueObject
						.getCOMPETITION_ID().length() > 0))) {
			sql = sql + "(OPPORTUNITY_ID = ? AND COMPETITION_ID = ?)";
			CaseNumber = 3;
		} else if (((valueObject.getOPPORTUNITY_ID() == null) || (valueObject
				.getOPPORTUNITY_ID().length() == 0))
				&& ((valueObject.getCFDA_NUMBER() != null) || (valueObject
						.getCFDA_NUMBER().length() > 0))
				&& ((valueObject.getCOMPETITION_ID() != null) || (valueObject
						.getCOMPETITION_ID().length() > 0))) {
			sql = sql + "(CFDA_NUMBER = ? AND COMPETITION_ID = ?)";
			CaseNumber = 4;
		} else if (((valueObject.getOPPORTUNITY_ID() == null) || (valueObject
				.getOPPORTUNITY_ID().length() == 0))
				&& ((valueObject.getCFDA_NUMBER() != null) || (valueObject
						.getCFDA_NUMBER().length() > 0))
				&& ((valueObject.getCOMPETITION_ID() == null) || (valueObject
						.getCOMPETITION_ID().length() == 0))) {
			sql = sql + "(CFDA_NUMBER = ?)";
			CaseNumber = 5;
		} else if (((valueObject.getOPPORTUNITY_ID() == null) || (valueObject
				.getOPPORTUNITY_ID().length() == 0))
				&& ((valueObject.getCFDA_NUMBER() == null) || (valueObject
						.getCFDA_NUMBER().length() == 0))
				&& ((valueObject.getCOMPETITION_ID() != null) || (valueObject
						.getCOMPETITION_ID().length() > 0))) {
			sql = sql + "(COMPETITION_ID = ?)";
			CaseNumber = 6;
		} else if (((valueObject.getOPPORTUNITY_ID() != null) || (valueObject
				.getOPPORTUNITY_ID().length() > 0))
				&& ((valueObject.getCFDA_NUMBER() != null) || (valueObject
						.getCFDA_NUMBER().length() > 0))
				&& ((valueObject.getCOMPETITION_ID() != null) || (valueObject
						.getCOMPETITION_ID().length() > 0))) {
			sql = sql
					+ "(OPPORTUNITY_ID = ? AND CFDA_NUMBER = ? AND COMPETITION_ID = ?)";
			CaseNumber = 7;
		}

		//==========================================================
		//  End of the Ending of the Dynamic "Where" Statement
		//==========================================================

		PreparedStatement stmt = null;

		try {
			stmt = conn.prepareStatement(sql);
			if (CaseNumber == 1)
				stmt.setString(1, valueObject.getOPPORTUNITY_ID());
			else if (CaseNumber == 2) {
				stmt.setString(1, valueObject.getOPPORTUNITY_ID());
				stmt.setString(2, valueObject.getCFDA_NUMBER());
			} else if (CaseNumber == 3) {
				stmt.setString(1, valueObject.getOPPORTUNITY_ID());
				stmt.setString(2, valueObject.getCOMPETITION_ID());
			} else if (CaseNumber == 4) {
				stmt.setString(1, valueObject.getCFDA_NUMBER());
				stmt.setString(2, valueObject.getCOMPETITION_ID());
			} else if (CaseNumber == 5)
				stmt.setString(1, valueObject.getCFDA_NUMBER());
			else if (CaseNumber == 6)
				stmt.setString(1, valueObject.getCOMPETITION_ID());
			else if (CaseNumber == 7) {
				stmt.setString(1, valueObject.getOPPORTUNITY_ID());
				stmt.setString(2, valueObject.getCFDA_NUMBER());
				stmt.setString(3, valueObject.getCOMPETITION_ID());
			}

			singleQuery(conn, stmt, valueObject);

		} finally {
			if (stmt != null)
				stmt.close();
		}
	}

	/**
	 * LoadAll-method. This will read all contents from database table and build
	 * an Vector containing valueObjects. Please note, that this method will
	 * consume huge amounts of resources if table has lot's of rows. This should
	 * only be used when target tables have only small amounts of data.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 */
	public List loadAll(Connection conn) throws SQLException {

		String sql = "SELECT * FROM getopportunity ORDER BY OPPORTUNITY_ID ASC ";
		List searchResults = listQuery(conn, conn.prepareStatement(sql));

		return searchResults;
	}

	/**
	 * create-method. This will create new row in database according to supplied
	 * valueObject contents. Make sure that values for all NOT NULL columns are
	 * correctly specified. Also, if this table does not use automatic
	 * surrogate-keys the primary-key must be specified. After INSERT command
	 * this method will read the generated primary-key back to valueObject if
	 * automatic surrogate-keys were used.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 * @param valueObject
	 *            This parameter contains the class instance to be created. If
	 *            automatic surrogate-keys are not used the Primary-key field
	 *            must be set for this to work properly.
	 */
	public synchronized void create(Connection conn, GetOpportunityList valueObject)
			throws SQLException {

		String sql = "";
		PreparedStatement stmt = null;
		ResultSet result = null;

		try {
			sql = "INSERT INTO getopportunity (  OPPORTUNITY_ID, OPPORTUNITY_TITLE, "
					+ "OPENING_DATE, CLOSING_DATE, CFDA_NUMBER, COMPETITION_ID"
					+ "SCHEMA_URL, INSTRUCTION_URL) VALUES (?, ?, ?, ?, ?, ?, ?, ?) ";
			stmt = conn.prepareStatement(sql);

			stmt.setInt(1, valueObject.getOID()); //-- auto incremented
			stmt.setString(1, valueObject.getOPPORTUNITY_ID());
			stmt.setString(2, valueObject.getOPPORTUNITY_TITLE());
			stmt.setLong(3, valueObject.getOPENING_DATE());
			stmt.setLong(4, valueObject.getCLOSING_DATE());
			stmt.setString(5, valueObject.getCFDA_NUMBER());
			stmt.setString(6, valueObject.getCOMPETITION_ID());
			stmt.setString(7, valueObject.getSCHEMA_URL());
			stmt.setString(8, valueObject.getINSTRUCTION_URL());

			int rowcount = databaseUpdate(conn, stmt);
			if (rowcount != 1) {
				//System.out.println("PrimaryKey Error when updating DB!");
				throw new SQLException("PrimaryKey Error when updating DB!");
			}

		} finally {
			if (stmt != null)
				stmt.close();
		}

	}

	/**
	 * save-method. This method will save the current state of valueObject to
	 * database. Save can not be used to create new instances in database, so
	 * upper layer must make sure that the primary-key is correctly specified.
	 * Primary-key will indicate which instance is going to be updated in
	 * database. If save can not find matching row, NotFoundException will be
	 * thrown.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 * @param valueObject
	 *            This parameter contains the class instance to be saved.
	 *            Primary-key field must be set for this to work properly.
	 */
	public void save(Connection conn, GetOpportunityList valueObject)
			throws NotFoundException, SQLException {

		String sql = "UPDATE getopportunity SET OPPORTUNITY_TITLE = ?, OPENING_DATE = ?, "
				+ "CLOSING_DATE = ?, CFDA_NUMBER = ?, COMPETITION_ID =?, SCHEMA_URL = ?, "
				+ "INSTRUCTION_URL = ? WHERE (OPPORTUNITY_ID  = ? ) ";
		PreparedStatement stmt = null;

		try {
			stmt = conn.prepareStatement(sql);

			stmt.setString(1, valueObject.getOPPORTUNITY_TITLE());
			stmt.setLong(2, valueObject.getOPENING_DATE());
			stmt.setLong(3, valueObject.getCLOSING_DATE());
			stmt.setString(4, valueObject.getCFDA_NUMBER());
			stmt.setString(5, valueObject.getCOMPETITION_ID());
			stmt.setString(6, valueObject.getSCHEMA_URL());
			stmt.setString(7, valueObject.getINSTRUCTION_URL());

			stmt.setString(8, valueObject.getOPPORTUNITY_ID());
			// stmt.setInt(8, valueObject.getOID());

			int rowcount = databaseUpdate(conn, stmt);
			if (rowcount == 0) {
				//System.out.println("Object could not be saved! (PrimaryKey
				// not found)");
				throw new NotFoundException(
						"Object could not be saved! (PrimaryKey not found)");
			}
			if (rowcount > 1) {
				//System.out.println("PrimaryKey Error when updating DB! (Many
				// objects were affected!)");
				throw new SQLException(
						"PrimaryKey Error when updating DB! (Many objects were affected!)");
			}
		} finally {
			if (stmt != null)
				stmt.close();
		}
	}

	/**
	 * delete-method. This method will remove the information from database as
	 * identified by by primary-key in supplied valueObject. Once valueObject
	 * has been deleted it can not be restored by calling save. Restoring can
	 * only be done using create method but if database is using automatic
	 * surrogate-keys, the resulting object will have different primary-key than
	 * what it was in the deleted object. If delete can not find matching row,
	 * NotFoundException will be thrown.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 * @param valueObject
	 *            This parameter contains the class instance to be deleted.
	 *            Primary-key field must be set for this to work properly.
	 */
	public void delete(Connection conn, GetOpportunityList valueObject)
			throws SQLException {

		String sql = "DELETE FROM getopportunity WHERE (OID = ? ) ";
		PreparedStatement stmt = null;

		try {
			stmt = conn.prepareStatement(sql);
			stmt.setInt(1, valueObject.getOID());

			int rowcount = databaseUpdate(conn, stmt);
			if (rowcount == 0) {
				//System.out.println("Object could not be deleted (PrimaryKey
				// not found)");
				throw new SQLException(
						"Object could not be deleted! (PrimaryKey not found)");
			}
			if (rowcount > 1) {
				//System.out.println("PrimaryKey Error when updating DB! (Many
				// objects were deleted!)");
				throw new SQLException(
						"PrimaryKey Error when updating DB! (Many objects were deleted!)");
			}
		} finally {
			if (stmt != null)
				stmt.close();
		}
	}

	/**
	 * deleteAll-method. This method will remove all information from the table
	 * that matches this Dao and ValueObject couple. This should be the most
	 * efficient way to clear table. Once deleteAll has been called, no
	 * valueObject that has been created before can be restored by calling save.
	 * Restoring can only be done using create method but if database is using
	 * automatic surrogate-keys, the resulting object will have different
	 * primary-key than what it was in the deleted object. (Note, the
	 * implementation of this method should be different with different DB
	 * backends.)
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 */
	public void deleteAll(Connection conn) throws SQLException {

		String sql = "DELETE FROM getopportunity";
		PreparedStatement stmt = null;

		try {
			stmt = conn.prepareStatement(sql);
			int rowcount = databaseUpdate(conn, stmt);
		} finally {
			if (stmt != null)
				stmt.close();
		}
	}

	/**
	 * coutAll-method. This method will return the number of all rows from table
	 * that matches this Dao. The implementation will simply execute "select
	 * count(primarykey) from table". If table is empty, the return value is 0.
	 * This method should be used before calling loadAll, to make sure table has
	 * not too many rows.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 */
	public int countAll(Connection conn) throws SQLException {

		String sql = "SELECT count(*) FROM getopportunity";
		PreparedStatement stmt = null;
		ResultSet result = null;
		int allRows = 0;

		try {
			stmt = conn.prepareStatement(sql);
			result = stmt.executeQuery();

			if (result.next())
				allRows = result.getInt(1);
		} finally {
			if (result != null)
				result.close();
			if (stmt != null)
				stmt.close();
		}
		return allRows;
	}

	/**
	 * searchMatching-Method. This method provides searching capability to get
	 * matching valueObjects from database. It works by searching all objects
	 * that match permanent instance variables of given object. Upper layer
	 * should use this by setting some parameters in valueObject and then call
	 * searchMatching. The result will be 0-N objects in vector, all matching
	 * those criteria you specified. Those instance-variables that have NULL
	 * values are excluded in search-criteria.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 * @param valueObject
	 *            This parameter contains the class instance where search will
	 *            be based. Primary-key field should not be set.
	 */
	public List searchMatching(Connection conn, GetOpportunityList valueObject)
			throws SQLException {

		List searchResults;

		boolean first = true;
		StringBuffer sql = new StringBuffer(
				"SELECT * FROM getopportunity WHERE 1=1 ");

		if ((valueObject.getOPPORTUNITY_ID() != null)
				&& (valueObject.getOPPORTUNITY_ID().length() > 0)) {
			if (first) {
				first = false;
			}
			sql.append("AND OPPORTUNITY_ID = '").append(
					valueObject.getOPPORTUNITY_ID()).append("' ");
		}

		if ((valueObject.getCFDA_NUMBER() != null)
				&& (valueObject.getCFDA_NUMBER().length() > 0)) {
			if (first) {
				first = false;
			}
			sql.append("AND CFDA_NUMBER = '").append(
					valueObject.getCFDA_NUMBER()).append("' ");
		}
		if ((valueObject.getCOMPETITION_ID() != null)
				&& (valueObject.getCOMPETITION_ID().length() > 0)) {
			if (first) {
				first = false;
			}
			sql.append("AND COMPETITION_ID = '").append(
					valueObject.getCOMPETITION_ID()).append("' ");
		}

		if (valueObject.getOPPORTUNITY_TITLE() != null) {
			if (first) {
				first = false;
			}
			sql.append("AND OPPORTUNITY_TITLE LIKE '").append(
					valueObject.getOPPORTUNITY_TITLE()).append("%' ");
		}

		if (valueObject.getOPENING_DATE() != 0) {
			if (first) {
				first = false;
			}
			sql.append("AND OPENING_DATE = ").append(
					valueObject.getOPENING_DATE()).append(" ");
		}

		if (valueObject.getCLOSING_DATE() != 0) {
			if (first) {
				first = false;
			}
			sql.append("AND CLOSING_DATE = ").append(
					valueObject.getCLOSING_DATE()).append(" ");
		}

		if (valueObject.getSCHEMA_URL() != null) {
			if (first) {
				first = false;
			}
			sql.append("AND SCHEMA_URL LIKE '").append(
					valueObject.getSCHEMA_URL()).append(" ");
		}

		if (valueObject.getINSTRUCTION_URL() != null) {
			if (first) {
				first = false;
			}
			sql.append("AND INSTRUCTION_URL LIKE '").append(
					valueObject.getINSTRUCTION_URL()).append("%' ");
		}

		sql.append("ORDER BY OPPORTUNITY_ID ASC ");

		// Prevent accidential full table results.
		// Use loadAll if all rows must be returned.
		if (first)
			searchResults = new Vector();
		else
			searchResults = listQuery(conn, conn.prepareStatement(sql
					.toString()));
		System.out.println("Search Results: " + searchResults);
		return searchResults;
	}

	/**
	 * getDaogenVersion will return information about generator which created
	 * these sources.
	 */
	public String getDaogenVersion() {
		return "DaoGen version 2.2.2";
	}

	/**
	 * databaseUpdate-method. This method is a helper method for internal use.
	 * It will execute all database handling that will change the information in
	 * tables. SELECT queries will not be executed here however. The return
	 * value indicates how many rows were affected. This method will also make
	 * sure that if cache is used, it will reset when data changes.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 * @param stmt
	 *            This parameter contains the SQL statement to be excuted.
	 */
	protected int databaseUpdate(Connection conn, PreparedStatement stmt)
			throws SQLException {

		int result = stmt.executeUpdate();

		return result;
	}

	/**
	 * databaseQuery-method. This method is a helper method for internal use. It
	 * will execute all database queries that will return only one row. The
	 * resultset will be converted to valueObject. If no rows were found,
	 * NotFoundException will be thrown.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 * @param stmt
	 *            This parameter contains the SQL statement to be excuted.
	 * @param valueObject
	 *            Class-instance where resulting data will be stored.
	 */
	protected void singleQuery(Connection conn, PreparedStatement stmt,
			GetOpportunityList valueObject) throws SQLException {

		ResultSet result = null;

		try {
			result = stmt.executeQuery();
			//System.out.println(result);

			if (result.next()) {

				valueObject.setOID(result.getInt("OID"));
				valueObject.setOPPORTUNITY_ID(result
						.getString("OPPORTUNITY_ID"));
				valueObject.setOPPORTUNITY_TITLE(result
						.getString("OPPORTUNITY_TITLE"));
				valueObject.setOPENING_DATE(result.getLong("OPENING_DATE"));
				valueObject.setCLOSING_DATE(result.getLong("CLOSING_DATE"));
				valueObject.setCFDA_NUMBER(result.getString("CFDA_NUMBER"));
				valueObject.setCOMPETITION_ID(result
						.getString("COMPETITION_ID"));
				valueObject.setSCHEMA_URL(result.getString("SCHEMA_URL"));
				valueObject.setINSTRUCTION_URL(result
						.getString("INSTRUCTION_URL"));
		

			} else {
				
				throw new SQLException("GetOpportunityList Object Not Found!");
			}
		} finally {
			if (result != null)
				result.close();
			if (stmt != null)
				stmt.close();
		}
	}

	/**
	 * databaseQuery-method. This method is a helper method for internal use. It
	 * will execute all database queries that will return multiple rows. The
	 * resultset will be converted to the List of valueObjects. If no rows were
	 * found, an empty List will be returned.
	 * 
	 * @param conn
	 *            This method requires working database connection.
	 * @param stmt
	 *            This parameter contains the SQL statement to be excuted.
	 */
	protected List listQuery(Connection conn, PreparedStatement stmt)
			throws SQLException {

		Vector searchResults = new Vector();
		ResultSet result = null;

		try {
			result = stmt.executeQuery();

			while (result.next()) {
				GetOpportunityList temp = new GetOpportunityList();

				temp.setOID(result.getInt("OID"));
				temp.setOPPORTUNITY_ID(result.getString("OPPORTUNITY_ID"));
				temp.setOPPORTUNITY_TITLE(result.getString("OPPORTUNITY_TITLE"));
				temp.setOPENING_DATE(result.getLong("OPENING_DATE"));
				temp.setCLOSING_DATE(result.getLong("CLOSING_DATE"));
				temp.setCFDA_NUMBER(result.getString("CFDA_NUMBER"));
				temp.setCOMPETITION_ID(result.getString("COMPETITION_ID"));
				temp.setSCHEMA_URL(result.getString("SCHEMA_URL"));
				temp.setINSTRUCTION_URL(result.getString("INSTRUCTION_URL"));

				searchResults.add(temp);
		

			}

		} finally {
			if (result != null)
				result.close();
			if (stmt != null)
				stmt.close();
		}

		return (List) searchResults;
	}

}

