/*
 * Created on Jun 28, 2004
 *
 */
package gov.grants.apply.util;

import gov.grants.apply.axis.util.AxisUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;

import org.apache.axis.attachments.AttachmentPart;
import org.apache.log4j.Logger;

import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.wswrapper.ByteArrayDataSource;

/**
 * Utility functions for S2S Server.
 * 
 * @author Brian Husted
 */
public class Utils {

	final static Logger log = Logger.getLogger(Utils.class.getName());

	private static Properties props = null;

	public static String PRODUCTION_MODE = "PRODUCTION_MODE";

	public static final String APP_PROP_FILE = "application.properties";

	public static final String ATTACHMENT_KEY = "ATTACHMENT_KEY";

	public static final String DATASOURCE_KEY = "DATASOURCE_KEY";

	/**
	 * Get a property value from the application properties file.
	 * 
	 * @param key
	 *            the String
	 * @return the property value or null if no property exists
	 * @throws IOException
	 */
	public static String getProperty(String key) throws IOException {
		if (props == null) {
			synchronized (Utils.class) {
				if (props == null) {
					props = new Properties();
					InputStream stream = null;
					try {
						String propertyFile = getPathToWEB_INF()
								+ APP_PROP_FILE;
						File f = new File(propertyFile);
						if (f.exists()) {
							stream = new FileInputStream(f);
						} else {
							stream = Utils.class
									.getResourceAsStream("/resources/"
											+ APP_PROP_FILE);
							log
									.debug("Loading property file from /resources/ directory");
						}
						props.load(stream);
						log.debug("Properties loaded successfully");

					} finally {
						try {
							stream.close();
						} catch (Exception ex) {
						}
					}
				}
			}

		}

		String value = props.getProperty(key);
		if (value == null) {
			value = "";
		}

		return value;

	}

	/**
	 * @return the string of the patch to WEB_INF.
	 */
	public static String getPathToWEB_INF() {
		String path = "";
		try {
			HttpServletRequest req = AxisUtils.getCurrentHttpRequest();
			path = req.getSession().getServletContext().getRealPath("/")
					.replace('\\', '/');
			path += "/" + "WEB-INF" + "/";
		} catch (Exception ex) {
		}
		return path;

	}

	/**
	 * Returns true if the server is being executed at Grants.gov. RI users
	 * should always set PRODUCTION_MODE property to false.
	 * 
	 * @return
	 */
	public static boolean isProductionMode() {
		try {
			if (Boolean
					.valueOf(Utils.getProperty(Utils.PRODUCTION_MODE).trim())
					.booleanValue()) {
				log.debug("Production Mode enabled.");
				return true;
			}
		} catch (Exception ex) {
			log.warn("Unable to determine production mode: defaulting "
					+ "to Production Mode!!", ex);
			return true;
		}
		return false;
	}

	/**
	 * Returns a Map with two Lists ByteArrayDataSource objects and DataSource
	 * objects. If we are using DIME then the large attachments can be buffered
	 * to a File without having the entire file in memory at once. This is the
	 * purpose for passing an Input Stream to the ByteArrayDataSource.
	 * 
	 * @return
	 * @throws Exception
	 */
	public static Map getSOAPAttachments() throws Exception {
		AttachmentPart[] attachments = AxisUtils.getMessageAttachments();
		log.debug("Received " + attachments.length + " attachments");
		Map map = new HashMap();
		List attachmentList = new ArrayList();
		List dataSourceList = new ArrayList();
		for (int i = 0; i < attachments.length; i++) {
			log.debug("Attachment Content Type:"
					+ attachments[i].getContentType());

			log
					.debug("Attachment Content-ID: "
							+ attachments[i].getContentId());

			InputStream in = attachments[i].getDataHandler().getInputStream();

			attachmentList.add(new ByteArrayDataSource(in, attachments[i]
					.getContentType(), attachments[i].getContentId()));
			dataSourceList.add(attachments[i].getDataHandler().getDataSource());
		}

		map.put(ATTACHMENT_KEY, attachmentList);
		map.put(DATASOURCE_KEY, dataSourceList);
		return map;
	}

}