package gov.grants.apply.valueObject;

import java.io.*;

/**
 * 
 * Application Value Object. This class is value object representing database
 * table application This class is intented to be used together with associated
 * Dao object.
 * 
 * @author Feng Qian
 */

public class Application implements Cloneable, Serializable {

	/**
	 * Persistent Instance variables. This data is directly mapped to the
	 * columns of database table.
	 */
	private int OID;
	private String TRACKING_ID;
	private String OPPORTUNITY_ID;
	private String CFDA_NUMBER;
	private String COMPETITION_ID;
	private long RECEIVED_DATE;
	private String STATUS;
	private long LAST_MODIFY_DATE;
	private String AGENCY_TRACKING_ID;
	private String ERROR_LOG;
	private String SUBMISSION_TITLE;

	/**
	 * Constructors. DaoGen generates two constructors by default. The first one
	 * takes no arguments and provides the most simple way to create object
	 * instance. The another one takes one argument, which is the primary key of
	 * the corresponding table.
	 */

	public Application() {

	}

	public Application(int OIDIn) {

		this.OID = OIDIn;

	}

	/**
	 * @return the int of OID.
	 */

	public int getOID() {
		return this.OID;
	}
	/**
	 * @param OIDIn
	 *            the int of OIDIn.
	 */
	public void setOID(int OIDIn) {
		this.OID = OIDIn;
	}

	/**
	 * @return the string of the tracking ID.
	 */
	public String getTRACKING_ID() {
		return this.TRACKING_ID;
	}
	/**
	 * @param TRACKING_IDIn
	 *            the string of the tracking ID.
	 */
	public void setTRACKING_ID(String TRACKING_IDIn) {
		this.TRACKING_ID = TRACKING_IDIn;
	}

	/**
	 * @return the string of the opportunity ID.
	 */
	public String getOPPORTUNITY_ID() {
		return this.OPPORTUNITY_ID;
	}
	/**
	 * @param OPPORTUNITY_IDIn
	 *            the string of the opportunity ID.
	 */
	public void setOPPORTUNITY_ID(String OPPORTUNITY_IDIn) {
		this.OPPORTUNITY_ID = OPPORTUNITY_IDIn;
	}

	/**
	 * @return the string of the CFDA number.
	 */
	public String getCFDA_NUMBER() {
		return this.CFDA_NUMBER;
	}
	/**
	 * @param CFDA_NUMBERIn
	 *            the string of the CFDA number.
	 */
	public void setCFDA_NUMBER(String CFDA_NUMBERIn) {
		this.CFDA_NUMBER = CFDA_NUMBERIn;
	}

	/**
	 * @return the string of the competition ID.
	 */
	public String getCOMPETITION_ID() {
		return this.COMPETITION_ID;
	}
	/**
	 * @param COMPETITION_IDIn
	 *            the string of the competition ID.
	 */
	public void setCOMPETITION_ID(String COMPETITION_IDIn) {
		this.COMPETITION_ID = COMPETITION_IDIn;
	}

	/**
	 * @return the long of the received date.
	 */
	public long getRECEIVED_DATE() {
		return this.RECEIVED_DATE;
	}
	/**
	 * @param RECEIVED_DATEIn
	 *            the long of the received date.
	 */
	public void setRECEIVED_DATE(long RECEIVED_DATEIn) {
		this.RECEIVED_DATE = RECEIVED_DATEIn;
	}

	/**
	 * @return the string of the status.
	 */
	public String getSTATUS() {
		return this.STATUS;
	}
	/**
	 * @param STATUSIn
	 *            the string of the status.
	 */
	public void setSTATUS(String STATUSIn) {
		this.STATUS = STATUSIn;
	}

	/**
	 * @return the string of the status.
	 */
	public String getERROR_LOG() {
		return this.ERROR_LOG;
	}
	/**
	 * @param STATUSIn
	 *            the string of the status.
	 */
	public void setERROR_LOG(String ERROR_LOGIn) {
		this.ERROR_LOG = ERROR_LOGIn;
	}

	/**
	 * @return the string of the SUBMISSION_TITLE.
	 */
	public String getSUBMISSION_TITLE() {
		return this.SUBMISSION_TITLE;
	}
	/**
	 * @param SUBMISSION_TITLEIn
	 *            the string of the SUBMISSION_TITLE.
	 */
	public void setSUBMISSION_TITLE(String SUBMISSION_TITLEIn) {
		this.SUBMISSION_TITLE = SUBMISSION_TITLEIn;
	}

	/**
	 * setAll allows to set all persistent variables in one method call. This is
	 * useful, when all data is available and it is needed to set the initial
	 * state of this object. Note that this method will directly modify instance
	 * variales, without going trough the individual set-methods.
	 * 
	 * @param OIDIn
	 *            the int.
	 * @param TRACKING_IDIn
	 *            the string.
	 * @param OPPORTUNITY_IDIn
	 *            the string.
	 * @param CFDA_NUMBERIn
	 *            the string.
	 * @param COMPETITION_IDIn
	 *            the string
	 * @param RECEIVED_DATEIn
	 *            the long.
	 * @param STATUSIn
	 *            the string.
	 * @param AGENCY_TRACKING_ID
	 *            the string.
	 * @param LAST_MODIFY_DATE
	 *            the long.
	 * @param ERROR_LOGIn
	 *            the string.
	 * @param SUBMISSION_TITLEIn
	 *            the string.
	 *  
	 */

	public void setAll(int OIDIn, String TRACKING_IDIn,
			String OPPORTUNITY_IDIn, String CFDA_NUMBERIn,
			String COMPETITION_IDIn, long RECEIVED_DATEIn, String STATUSIn,
			String AGENCY_TRACKING_ID, long LAST_MODIFY_DATE,
			String ERROR_LOGIn, String SUBMISSION_TITLEIn) {
		this.OID = OIDIn;
		this.TRACKING_ID = TRACKING_IDIn;
		this.OPPORTUNITY_ID = OPPORTUNITY_IDIn;
		this.CFDA_NUMBER = CFDA_NUMBERIn;
		this.COMPETITION_ID = COMPETITION_IDIn;
		this.RECEIVED_DATE = RECEIVED_DATEIn;
		this.STATUS = STATUSIn;
		this.AGENCY_TRACKING_ID = AGENCY_TRACKING_ID;
		this.LAST_MODIFY_DATE = LAST_MODIFY_DATE;
		this.ERROR_LOG = ERROR_LOGIn;
		this.SUBMISSION_TITLE = SUBMISSION_TITLEIn;
	}

	/**
	 * @param valueObject
	 *            the Application object.
	 * @return a true if two Application instances contain same values in all
	 *         persistent instance variables. If hasEqualMapping returns true,
	 *         it does not mean the objects are the same instance. However it
	 *         does mean that in that moment, they are mapped to the same row in
	 *         database. Otherwise return false.
	 */
	public boolean hasEqualMapping(Application valueObject) {

		if (valueObject.getOID() != this.OID) {
			return (false);
		}
		if (this.TRACKING_ID == null) {
			if (valueObject.getTRACKING_ID() != null)
				return (false);
		} else if (!this.TRACKING_ID.equals(valueObject.getTRACKING_ID())) {
			return (false);
		}
		if (this.OPPORTUNITY_ID == null) {
			if (valueObject.getOPPORTUNITY_ID() != null)
				return (false);
		} else if (!this.OPPORTUNITY_ID.equals(valueObject.getOPPORTUNITY_ID())) {
			return (false);
		}
		if (this.CFDA_NUMBER == null) {
			if (valueObject.getCFDA_NUMBER() != null)
				return (false);
		} else if (!this.CFDA_NUMBER.equals(valueObject.getCFDA_NUMBER())) {
			return (false);
		}
		if (this.COMPETITION_ID == null) {
			if (valueObject.getCOMPETITION_ID() != null)
				return (false);
		} else if (!this.COMPETITION_ID.equals(valueObject.getCOMPETITION_ID())) {
			return (false);
		}
		if (valueObject.getRECEIVED_DATE() != this.RECEIVED_DATE) {
			return (false);
		}
		if (this.STATUS == null) {
			if (valueObject.getSTATUS() != null)
				return (false);
		} else if (!this.STATUS.equals(valueObject.getSTATUS())) {
			return (false);
		}
		if (this.ERROR_LOG == null) {
			if (valueObject.getERROR_LOG() != null)
				return (false);
		} else if (!this.ERROR_LOG.equals(valueObject.getERROR_LOG())) {
			return (false);
		}
		if (this.SUBMISSION_TITLE == null) {
			if (valueObject.getSUBMISSION_TITLE() != null)
				return (false);
		} else if (!this.SUBMISSION_TITLE.equals(valueObject
				.getSUBMISSION_TITLE())) {
			return (false);
		}

		return true;
	}

	/**
	 * toString will return String object representing the state of this
	 * valueObject. This is useful during application development, and possibly
	 * when application is writing object states in textlog.
	 */
	public String toString() {
		StringBuffer out = new StringBuffer(this.getDaogenVersion());
		out.append("\nclass Application, mapping to table application\n");
		out.append("Persistent attributes: \n");
		out.append("OID = " + this.OID + "\n");
		out.append("TRACKING_ID = " + this.TRACKING_ID + "\n");
		out.append("OPPORTUNITY_ID = " + this.OPPORTUNITY_ID + "\n");
		out.append("CFDA_NUMBER = " + this.CFDA_NUMBER + "\n");
		out.append("COMPETITION_ID = " + this.COMPETITION_ID + "\n");
		out.append("RECEIVED_DATE = " + this.RECEIVED_DATE + "\n");
		out.append("STATUS = " + this.STATUS + "\n");
		out.append("AGENCY_TRACKING_ID = " + this.AGENCY_TRACKING_ID + "\n");
		out.append("LAST_MODIFY_DATE = " + this.LAST_MODIFY_DATE);
		out.append("ERROR_LOG = " + this.ERROR_LOG);
		out.append("SUBMISSION_TITLE = " + this.SUBMISSION_TITLE);
		return out.toString();
	}

	/**
	 * Clone will return identical deep copy of this valueObject. Note, that
	 * this method is different than the clone() which is defined in
	 * java.lang.Object. Here, the retuned cloned object will also have all its
	 * attributes cloned.
	 */
	public Object clone() {
		Application cloned = new Application();

		cloned.setOID(this.OID);
		cloned.setTRACKING_ID(new String(this.TRACKING_ID));
		cloned.setOPPORTUNITY_ID(new String(this.OPPORTUNITY_ID));
		cloned.setCFDA_NUMBER(new String(this.CFDA_NUMBER));
		cloned.setCOMPETITION_ID(new String(this.COMPETITION_ID));
		cloned.setRECEIVED_DATE(this.RECEIVED_DATE);
		cloned.setSTATUS(new String(this.STATUS));
		cloned.setAGENCY_TRACKING_ID(new String(this.AGENCY_TRACKING_ID));
		cloned.setLAST_MODIFY_DATE(this.LAST_MODIFY_DATE);
		cloned.setERROR_LOG(this.ERROR_LOG);
		cloned.setSUBMISSION_TITLE(this.SUBMISSION_TITLE);

		return cloned;
	}

	/**
	 * getDaogenVersion will return information about generator which created
	 * these sources.
	 */
	public String getDaogenVersion() {
		return "DaoGen version 2.2.2";
	}

	/**
	 * @return Returns the aGENCY_TRACKING_ID.
	 */
	public String getAGENCY_TRACKING_ID() {
		return AGENCY_TRACKING_ID;
	}
	/**
	 * @param agency_tracking_id
	 *            The aGENCY_TRACKING_ID.
	 */
	public void setAGENCY_TRACKING_ID(String agency_tracking_id) {
		AGENCY_TRACKING_ID = agency_tracking_id;
	}
	/**
	 * @return Returns the lAST_MODIFY_DATE.
	 */
	public long getLAST_MODIFY_DATE() {
		return LAST_MODIFY_DATE;
	}
	/**
	 * @param last_modify_date
	 *            The lAST_MODIFY_DATE.
	 */
	public void setLAST_MODIFY_DATE(long last_modify_date) {
		LAST_MODIFY_DATE = last_modify_date;
	}
}

