package gov.grants.apply.valueObject;

import java.io.*;


 /**

  * GetOpportunity Value Object.
  * This class is value object representing database table getopportunity
  * This class is intented to be used together with associated Dao object.
  * 
  * @author Enoch Moses
  *
  */





public class GetOpportunityList implements Cloneable, Serializable {

    /**
     * Persistent Instance variables. This data is directly
     * mapped to the columns of database table.
     */
    private int OID;
    private String OPPORTUNITY_ID;
    private String OPPORTUNITY_TITLE;
    private long OPENING_DATE;
    private long CLOSING_DATE;
    private String CFDA_NUMBER;
    private String COMPETITION_ID;
    private String SCHEMA_URL;
    private String INSTRUCTION_URL;



    /**
     * Constructors. DaoGen generates two constructors by default.
     * The first one takes no arguments and provides the most simple
     * way to create object instance. The another one takes one
     * argument, which is the primary key of the corresponding table.
     */

    public GetOpportunityList () {

    }

    /**
	 * @param OPPORTUNITY_IDIn the string of Opportunity ID.
	 */
	public GetOpportunityList (String OPPORTUNITY_IDIn) {

		this.OPPORTUNITY_ID = OPPORTUNITY_IDIn;

    }


    /**
	 * @return the int of OID.
	 */
	public int getOID() {
          return this.OID;
    }
    /**
	 * @param OIDIn the int of OID.
	 */
	public void setOID(int OIDIn) {
          this.OID = OIDIn;
    }

    /**
	 * @return the string of the opportunity ID.
	 */
	public String getOPPORTUNITY_ID() {
          return this.OPPORTUNITY_ID;
    }
    /**
	 * @param OPPORTUNITY_IDIn the string of the opportunity ID.
	 */
	public void setOPPORTUNITY_ID(String OPPORTUNITY_IDIn) {
          this.OPPORTUNITY_ID = OPPORTUNITY_IDIn;
    }

    /**
	 * @return the string of the opportunity title.
	 */
	public String getOPPORTUNITY_TITLE() {
          return this.OPPORTUNITY_TITLE;
    }
    /**
	 * @param OPPORTUNITY_TITLEIn the string of the opportunity title.
	 */
	public void setOPPORTUNITY_TITLE(String OPPORTUNITY_TITLEIn) {
          this.OPPORTUNITY_TITLE = OPPORTUNITY_TITLEIn;
    }

    /**
	 * @return the long of the opening date.
	 */
	public long getOPENING_DATE() {
          return this.OPENING_DATE;
    }
    /**
	 * @param OPENING_DATEIn the long of the opening date.
	 */
	public void setOPENING_DATE(long OPENING_DATEIn) {
          this.OPENING_DATE = OPENING_DATEIn;
    }

    /**
	 * @return the long of the closing date.
	 */
	public long getCLOSING_DATE() {
          return this.CLOSING_DATE;
    }
    /**
	 * @param CLOSING_DATEIn the long of the closing date.
	 */
	public void setCLOSING_DATE(long CLOSING_DATEIn) {
          this.CLOSING_DATE = CLOSING_DATEIn;
    }

    /**
	 * @return the string of the CFDA number.
	 */
	public String getCFDA_NUMBER() {
          return this.CFDA_NUMBER;
    }
    /**
	 * @param CFDA_NUMBERIn the string of the CFDA number.
	 */
	public void setCFDA_NUMBER(String CFDA_NUMBERIn) {
          this.CFDA_NUMBER = CFDA_NUMBERIn;
    }

    /**
	 * @return the string of the competition ID.
	 */
	public String getCOMPETITION_ID(){
    	 return this.COMPETITION_ID;
    }
    /**
	 * @param COMPETITION_IDIn the string of the competition ID.
	 */
	public void setCOMPETITION_ID(String COMPETITION_IDIn) {
    	this.COMPETITION_ID = COMPETITION_IDIn;
    }
    /**
	 * @return the string of the schema URL.
	 */
	public String getSCHEMA_URL() {
          return this.SCHEMA_URL;
    }
    /**
	 * @param SCHEMA_URLIn the string of the schema URL.
	 */
	public void setSCHEMA_URL(String SCHEMA_URLIn) {
          this.SCHEMA_URL = SCHEMA_URLIn;
    }

    /**
	 * @return the string of the instruction URL.
	 */
	public String getINSTRUCTION_URL() {
          return this.INSTRUCTION_URL;
    }
    /**
	 * @param INSTRUCTION_URLIn the string of the instruction URL.
	 */
	public void setINSTRUCTION_URL(String INSTRUCTION_URLIn) {
          this.INSTRUCTION_URL = INSTRUCTION_URLIn;
    }



    /**
     * setAll allows to set all persistent variables in one method call.
     * This is useful, when all data is available and it is needed to
     * set the initial state of this object. Note that this method will
     * directly modify instance variales, without going trough the
     * individual set-methods.
     * 
     * @param OIDIn the String.
	 * @param OPPORTUNITY_IDIn the String.
	 * @param OPPORTUNITY_TITLEIn the String.
	 * @param OPENING_DATEIn the long.
	 * @param CLOSING_DATEIn the long.
	 * @param CFDA_NUMBERIn the String.
	 * @param SCHEMA_URLIn the String.
	 * @param INSTRUCTION_URLIn the String.
     */

    public void setAll(int OIDIn,
          String OPPORTUNITY_IDIn,
          String OPPORTUNITY_TITLEIn,
          long OPENING_DATEIn,
          long CLOSING_DATEIn,
          String CFDA_NUMBERIn,
          String SCHEMA_URLIn,
          String INSTRUCTION_URLIn) {
          this.OID = OIDIn;
          this.OPPORTUNITY_ID = OPPORTUNITY_IDIn;
          this.OPPORTUNITY_TITLE = OPPORTUNITY_TITLEIn;
          this.OPENING_DATE = OPENING_DATEIn;
          this.CLOSING_DATE = CLOSING_DATEIn;
          this.CFDA_NUMBER = CFDA_NUMBERIn;
          this.SCHEMA_URL = SCHEMA_URLIn;
          this.INSTRUCTION_URL = INSTRUCTION_URLIn;
    }


    /**
	 * @param valueObject the GetOpportunityList.
	 * @return a true if they contain same values in all persistent instance
     * variables. If hasEqualMapping returns true, it does not mean the objects
     * are the same instance. However it does mean that in that moment, they
     * are mapped to the same row in database. A false if otherwise.
     */
    public boolean hasEqualMapping(GetOpportunityList valueObject) {

          if (valueObject.getOID() != this.OID) {
                    return(false);
          }
		  if (valueObject.getOPPORTUNITY_ID() != this.OPPORTUNITY_ID) {
						 return(false);
		  }
          if (this.OPPORTUNITY_TITLE == null) {
                    if (valueObject.getOPPORTUNITY_TITLE() != null)
                           return(false);
          } else if (!this.OPPORTUNITY_TITLE.equals(valueObject.getOPPORTUNITY_TITLE())) {
                    return(false);
          }
          if (valueObject.getOPENING_DATE() != this.OPENING_DATE) {
                    return(false);
          }
          if (valueObject.getCLOSING_DATE() != this.CLOSING_DATE) {
                    return(false);
          }
          if (this.CFDA_NUMBER == null) {
                    if (valueObject.getCFDA_NUMBER() != null)
                           return(false);
          } else if (!this.CFDA_NUMBER.equals(valueObject.getCFDA_NUMBER())) {
                    return(false);
          }
          if (this.SCHEMA_URL == null) {
                    if (valueObject.getSCHEMA_URL() != null)
                           return(false);
          } else if (!this.SCHEMA_URL.equals(valueObject.getSCHEMA_URL())) {
                    return(false);
          }
          if (this.INSTRUCTION_URL == null) {
                    if (valueObject.getINSTRUCTION_URL() != null)
                           return(false);
          } else if (!this.INSTRUCTION_URL.equals(valueObject.getINSTRUCTION_URL())) {
                    return(false);
          }

          return true;
    }



    /**
     * toString will return String object representing the state of this
     * valueObject. This is useful during application development, and
     * possibly when application is writing object states in textlog.
     */
    public String toString() {
        StringBuffer out = new StringBuffer(this.getDaogenVersion());
        out.append("\nclass GetOpportunityListList, mapping to table getopportunity\n");
        out.append("Persistent attributes: \n");
        out.append("OID = " + this.OID + "\n");
        out.append("OPPORTUNITY_ID = " + this.OPPORTUNITY_ID + "\n");
        out.append("OPPORTUNITY_TITLE = " + this.OPPORTUNITY_TITLE + "\n");
        out.append("OPENING_DATE = " + this.OPENING_DATE + "\n");
        out.append("CLOSING_DATE = " + this.CLOSING_DATE + "\n");
        out.append("CFDA_NUMBER = " + this.CFDA_NUMBER + "\n");
        out.append("SCHEMA_URL = " + this.SCHEMA_URL + "\n");
        out.append("INSTRUCTION_URL = " + this.INSTRUCTION_URL + "\n");
        return out.toString();
    }


    /**
     * Clone will return identical deep copy of this valueObject.
     * Note, that this method is different than the clone() which
     * is defined in java.lang.Object. Here, the retuned cloned object
     * will also have all its attributes cloned.
     */
    public Object clone() {
        GetOpportunityList cloned = new GetOpportunityList();

        cloned.setOID(this.OID);
        cloned.setOPPORTUNITY_ID(this.OPPORTUNITY_ID);
        cloned.setOPPORTUNITY_TITLE(new String(this.OPPORTUNITY_TITLE));
        cloned.setOPENING_DATE(this.OPENING_DATE);
        cloned.setCLOSING_DATE(this.CLOSING_DATE);
        cloned.setCFDA_NUMBER(new String(this.CFDA_NUMBER));
        cloned.setSCHEMA_URL(new String(this.SCHEMA_URL));
        cloned.setINSTRUCTION_URL(new String(this.INSTRUCTION_URL));


        return cloned;
    }



    /**
	 * @return the string of the DaoGen Version.
     */
    public String getDaogenVersion() {
        return "DaoGen version 2.2.2";
    }

}

