/*
 * Created on Jun 24, 2004
 */
package gov.grants.apply.ws;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicationInformationType;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.GrantsGovApplicationStatusType;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest_ApplicationFilter;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest_ApplicationFilter_Filter;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListResponse;
import gov.grants.apply.system.Global_V1_0.StringMin1Max100Type;
import gov.grants.apply.system.Global_V1_0.StringMin1Max15Type;
import gov.grants.apply.system.Global_V1_0.StringMin1Max240Type;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.GGWSWrapper;
import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.wswrapper.ApplicationListInfo;
import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.wswrapper.FilterType;
import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.wswrapper.SearchFilter;

/**
 * This is the class for Processing a getting application list web service
 * request and obtaining a list of applications.
 * 
 * @author Brian Husted
 *  
 */
public class GetApplicationListProcessor extends AbstractWSProcessor {

	static final Logger log = Logger
			.getLogger(GetApplicationListProcessor.class.getName());

	/**
	 * Processes a GetApplicationListRequest.
	 * 
	 * @param request
	 *            the Object of the webservice request.
	 * @return the Object of the applications.
	 */
	public Object processRequest(Object request) throws Exception {

		_GetApplicationListResponse response = new _GetApplicationListResponse();

		//Get the matching applications
		List applicationFilters = getApplicationFilters((_GetApplicationListRequest) request);
		
		List applications = getApplicationList(applicationFilters);
		//Format the response
		List infoTypeList = buildResponse(applications);
		//set the applications into the response
		response
				.setApplicationInformation((ApplicationInformationType[]) infoTypeList
						.toArray(new ApplicationInformationType[0]));

		return response;
	}

	protected List getApplicationList(List applicationFilters) throws Exception {

		return GGWSWrapper.getAORApplicationList(getUserId(),
				applicationFilters);

	}

	private static List buildResponse(List list) {
		List appInfoList = new ArrayList();
		if (list != null) {
			log.debug("The query returned " + list.size() + " rows.");
			Iterator i = list.iterator();

			while (i.hasNext()) {
				ApplicationListInfo application = (ApplicationListInfo) i
						.next();

				ApplicationInformationType applicationInformationType = new ApplicationInformationType();
				if (application.getCfdaNumber() != null) {
					applicationInformationType
							.setCFDANumber(new StringMin1Max15Type(application
									.getCfdaNumber()));
				}
				if (application.getCompetitionId() != null) {
					applicationInformationType
							.setCompetitionID(new StringMin1Max100Type(
									application.getCompetitionId()));
				}
				if (application.getTrackingNumber() != null) {
					applicationInformationType
							.setGrants_govTrackingNumber(new StringMin1Max240Type(
									application.getTrackingNumber()));
				}
				if (application.getStatus() != null) {
					applicationInformationType
							.setGrantsGovApplicationStatus(GrantsGovApplicationStatusType
									.fromString(application.getStatus()));
				}
				if (application.getTimestamp() != null) {
					Calendar c = Calendar.getInstance();
					c.setTime(application.getTimestamp());
					applicationInformationType.setStatusDateTime(c);
				}

				if (application.getOpportunityNumber() != null) {
					applicationInformationType
							.setOpportunityID(new StringMin1Max100Type(
									application.getOpportunityNumber()));
				}

				if (application.getReceivedDate() != null) {
					Calendar c = Calendar.getInstance();
					c.setTime(application.getReceivedDate());
					applicationInformationType.setReceivedDateTime(c);
				}

				if (application.getAgencyTrackingId() != null) {
					applicationInformationType
							.setAgencyTrackingNumber(new StringMin1Max240Type(
									application.getAgencyTrackingId()));
				}

				if (application.getSubmissionTitle() != null) {
					applicationInformationType
							.setSubmissionTitle(new StringMin1Max240Type(
									application.getSubmissionTitle()));

				}

				appInfoList.add(applicationInformationType);
			}
		}
		return appInfoList;
	}

	private static List getApplicationFilters(_GetApplicationListRequest request)
			throws Exception {

		List filters = new ArrayList();

		_GetApplicationListRequest_ApplicationFilter[] applicationFilters = request
				.getApplicationFilter();
		if (applicationFilters != null) {
			for (int i = 0; i < applicationFilters.length; i++) {
				if (applicationFilters[i].getFilter() != null) {
					if (_GetApplicationListRequest_ApplicationFilter_Filter._Status
							.equals(applicationFilters[i].getFilter()
									.getValue())) {
						log.debug("Added a status filter of: "
								+ applicationFilters[i].getFilterValue());
						String status = applicationFilters[i].getFilterValue()
								.trim();
						if (status.length() > 0) {
							SearchFilter statusFilter = new SearchFilter(
									FilterType.STATUS, status);
							filters.add(statusFilter);
						}
					}

					if (_GetApplicationListRequest_ApplicationFilter_Filter._CFDANumber
							.equals(applicationFilters[i].getFilter()
									.getValue())) {
						log.debug("Added a cfda filter of: "
								+ applicationFilters[i].getFilterValue());
						String cfdaNumber = applicationFilters[i]
								.getFilterValue().trim();
						if (cfdaNumber.length() > 0) {
							SearchFilter cfdaFilter = new SearchFilter(
									FilterType.CFDA, cfdaNumber);
							filters.add(cfdaFilter);
						}
					}

					if (_GetApplicationListRequest_ApplicationFilter_Filter._OpportunityID
							.equals(applicationFilters[i].getFilter()
									.getValue())) {
						log.debug("Added an oppId filter of: "
								+ applicationFilters[i].getFilterValue());
						String oppID = applicationFilters[i].getFilterValue()
								.trim();
						if (oppID.length() > 0) {
							SearchFilter oppIdFilter = new SearchFilter(
									FilterType.OPPNUM, oppID);
							filters.add(oppIdFilter);
						}
					}

					if (_GetApplicationListRequest_ApplicationFilter_Filter._SubmissionTitle
							.equals(applicationFilters[i].getFilter()
									.getValue())) {
						log.debug("Added a submission title filter of: "
								+ applicationFilters[i].getFilterValue());
						String submissionTitle = applicationFilters[i]
								.getFilterValue().trim();
						if (submissionTitle.length() > 0) {
							SearchFilter subTitleFilter = new SearchFilter(
									FilterType.SUBMISSION_TITLE,
									submissionTitle);
							filters.add(subTitleFilter);
						}
					}

				}
			}
		}

		return filters;
	}
}