/*
 * Created on Jun 24, 2004
 */
package gov.grants.apply.ws;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicationInformationType;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.GrantsGovApplicationStatusType;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest_ApplicationFilter;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest_ApplicationFilter_Filter;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListResponse;
import gov.grants.apply.dao.ApplicationDao;
import gov.grants.apply.system.Global_V1_0.StringMin1Max100Type;
import gov.grants.apply.system.Global_V1_0.StringMin1Max15Type;
import gov.grants.apply.system.Global_V1_0.StringMin1Max240Type;
import gov.grants.apply.util.DBUtils;
import gov.grants.apply.valueObject.Application;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * This is the class for Processing a getting application list web service
 * request and obtaining a list of applications from the database.
 * 
 * @author Feng Qian
 *  
 */
public class GetApplicationListProcessor_RI extends AbstractWSProcessor_RI {

	static final Logger log = Logger
			.getLogger(GetApplicationListProcessor_RI.class.getName());

	/**
	 * @param request
	 *            the Object of the webservice request.
	 * @return the Object of the applications.
	 */
	public Object processRequest(Object request) throws Exception {

		_GetApplicationListResponse response = new _GetApplicationListResponse();

		//Get the matching applications
		List applications = getApplications((_GetApplicationListRequest) request);
		//Format the response
		List infoTypeList = buildResponse(applications);
		//set the applications into the response
		response
				.setApplicationInformation((ApplicationInformationType[]) infoTypeList
						.toArray(new ApplicationInformationType[0]));

		return response;
	}

	/**
	 * @param list
	 *            the List of the Application object.
	 * @return the List of the applications.
	 */
	protected List buildResponse(List list) {
		List appInfoList = new ArrayList();
		if (list != null) {
			log.debug("The query returned " + list.size() + " rows.");
			Iterator i = list.iterator();

			while (i.hasNext()) {
				Application application = (Application) i.next();

				ApplicationInformationType applicationInformationType = new ApplicationInformationType();
				if (application.getCFDA_NUMBER() != null) {
					applicationInformationType
							.setCFDANumber(new StringMin1Max15Type(application
									.getCFDA_NUMBER()));
				}
				if (application.getCOMPETITION_ID() != null) {
					applicationInformationType
							.setCompetitionID(new StringMin1Max100Type(
									application.getCOMPETITION_ID()));
				}
				if (application.getTRACKING_ID() != null) {
					applicationInformationType
							.setGrants_govTrackingNumber(new StringMin1Max240Type(
									application.getTRACKING_ID()));
				}
				if (application.getSTATUS() != null) {
					applicationInformationType
							.setGrantsGovApplicationStatus(GrantsGovApplicationStatusType
									.fromString(application.getSTATUS()));
				}
				if (application.getLAST_MODIFY_DATE() > 0) {
					Calendar c = Calendar.getInstance();
					c.setTime(new Date(application.getLAST_MODIFY_DATE()));
					applicationInformationType.setStatusDateTime(c);
				}

				if (application.getOPPORTUNITY_ID() != null) {
					applicationInformationType
							.setOpportunityID(new StringMin1Max100Type(
									application.getOPPORTUNITY_ID()));
				}
				if (application.getRECEIVED_DATE() > 0) {
					Calendar c = Calendar.getInstance();
					c.setTime(new Date(application.getRECEIVED_DATE()));
					applicationInformationType.setReceivedDateTime(c);
				}
				if (application.getAGENCY_TRACKING_ID() != null) {
					applicationInformationType
							.setAgencyTrackingNumber(new StringMin1Max240Type(
									application.getAGENCY_TRACKING_ID()));

				}
				if (application.getSUBMISSION_TITLE() != null) {
					applicationInformationType
							.setSubmissionTitle(new StringMin1Max240Type(
									application.getSUBMISSION_TITLE()));

				}

				appInfoList.add(applicationInformationType);
			}
		}
		return appInfoList;
	}

	/**
	 * @param request
	 *            the _GetApplicationListRequest of the getting application list
	 *            webservice request.
	 * @return the List of the applications.
	 * @throws Exception
	 */
	protected List getApplications(_GetApplicationListRequest request)
			throws Exception {

		Connection conn = null;
		Application application = null;
		List applications = null;

		try {
			conn = DBUtils.getConnection();

			String status = null;
			String CFDANumber = null;
			String opportunityID = null;
			String submissionTitle = null;
			_GetApplicationListRequest_ApplicationFilter[] applicationFilters = request
					.getApplicationFilter();
			if (applicationFilters != null) {
				for (int i = 0; i < applicationFilters.length; i++) {
					if (applicationFilters[i].getFilter() != null) {
						if (_GetApplicationListRequest_ApplicationFilter_Filter._Status
								.equals(applicationFilters[i].getFilter()
										.getValue())) {
							log.debug("Added a status filter of: "
									+ applicationFilters[i].getFilterValue());
							status = applicationFilters[i].getFilterValue()
									.trim();
						}

						if (_GetApplicationListRequest_ApplicationFilter_Filter._CFDANumber
								.equals(applicationFilters[i].getFilter()
										.getValue())) {
							log.debug("Added a cfda filter of: "
									+ applicationFilters[i].getFilterValue());
							CFDANumber = applicationFilters[i].getFilterValue()
									.trim();
						}

						if (_GetApplicationListRequest_ApplicationFilter_Filter._OpportunityID
								.equals(applicationFilters[i].getFilter()
										.getValue())) {
							log.debug("Added an oppId filter of: "
									+ applicationFilters[i].getFilterValue());
							opportunityID = applicationFilters[i]
									.getFilterValue().trim();
						}

						if (_GetApplicationListRequest_ApplicationFilter_Filter._SubmissionTitle
								.equals(applicationFilters[i].getFilter()
										.getValue())) {
							log.debug("Added a submission title filter of: "
									+ applicationFilters[i].getFilterValue());
							submissionTitle = applicationFilters[i]
									.getFilterValue().trim();
						}

					}
				}
			}

			application = new Application();
			application.setSTATUS(status);
			application.setCFDA_NUMBER(CFDANumber);
			application.setOPPORTUNITY_ID(opportunityID);
			application.setSUBMISSION_TITLE(submissionTitle);
			applications = new ApplicationDao().searchMatching(conn,
					application);

		} finally {
			try {
				conn.close();
			} catch (Exception ex) {
			}
		}
		return applications;
	}
}