
package gov.grants.apply.ws;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationStatusDetailRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationStatusDetailResponse;
import gov.grants.apply.system.Global_V1_0.StringMin1Max240Type;

import org.apache.log4j.Logger;

import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.GGWSWrapper;

/**
 * This is the class for Processing a getting application status detail web
 * service request and obtaining a detailed application status for a particular
 * application.
 * 
 * @author Brian Husted
 *  
 */
public class GetApplicationStatusDetailProcessor extends AbstractWSProcessor {

	static final Logger log = Logger
			.getLogger(GetApplicationStatusDetailProcessor.class.getName());

	/**
	 * @param request
	 *            the Object of the webservice request.
	 * @return the Object of the application status detail.
	 */
	public Object processRequest(Object request) throws Exception {

		//Create a response object
		_GetApplicationStatusDetailResponse response = new _GetApplicationStatusDetailResponse();
		
		_GetApplicationStatusDetailRequest appStatusDetailRequest = (_GetApplicationStatusDetailRequest) request;
		
		String grants_govTrackingNumber = appStatusDetailRequest
		.getGrants_govTrackingNumber().toString();
				
		String statusDetail = getStatusDetail(grants_govTrackingNumber);
		
		
		if (statusDetail == null ) {
			statusDetail = "No additional status information exists for GRANT: "
							+ grants_govTrackingNumber;
		}
		
		response.setDetailedStatus( statusDetail );
		
		//set the tracking number into the response
		response.setGrants_govTrackingNumber(new StringMin1Max240Type(
				grants_govTrackingNumber));

		return response;
	}

	/**
	 * @param String
	 *            the Grants.gov tracking number from the webservice request.
	 * @return the Application object that contains the detailed status.
	 * @throws Exception
	 */
	protected String getStatusDetail(String ggTrackingID) throws Exception {

		return GGWSWrapper.getStatusDetails( getUserId(), ggTrackingID );
	}
}