/*
 * Created on November 4, 2004
 */
package gov.grants.apply.ws;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationStatusDetailRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationStatusDetailResponse;
import gov.grants.apply.dao.ApplicationDao;
import gov.grants.apply.system.Global_V1_0.StringMin1Max240Type;
import gov.grants.apply.util.DBUtils;
import gov.grants.apply.valueObject.Application;

import java.sql.Connection;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * This is the class for Processing a getting application status detail web
 * service request and obtaining a detailed application status from the
 * database.
 * 
 * @author Mark Sommer
 *  
 */
public class GetApplicationStatusDetailProcessor_RI extends AbstractWSProcessor_RI {

	static final Logger log = Logger
			.getLogger(GetApplicationStatusDetailProcessor_RI.class.getName());

	/**
	 * @param request
	 *            the Object of the webservice request.
	 * @return the Object of the application status detail.
	 */
	public Object processRequest(Object request) throws Exception {

		//Create a response object
		_GetApplicationStatusDetailResponse response = new _GetApplicationStatusDetailResponse();
		
		_GetApplicationStatusDetailRequest appStatusDetailRequest = (_GetApplicationStatusDetailRequest) request;
		
		String grants_govTrackingNumber = appStatusDetailRequest
		.getGrants_govTrackingNumber().toString();
				
		Application application = getStatusDetail(grants_govTrackingNumber);
		
		if (application.getERROR_LOG() == null
				|| application.getERROR_LOG().toString().length() <= 0) {
			application
					.setERROR_LOG(  "No additional status information exists for GRANT: "
							+ grants_govTrackingNumber);
		}
		response.setDetailedStatus( application.getERROR_LOG() );
		//set the tracking number into the response
		response.setGrants_govTrackingNumber(new StringMin1Max240Type(
				grants_govTrackingNumber));

		return response;
	}

	/**
	 * @param String
	 *            the Grants.gov tracking number from the webservice request.
	 * @return the Application object that contains the detailed status.
	 * @throws Exception
	 */
	protected Application getStatusDetail(String ggTrackingID) throws Exception {

		Connection conn = null;
		Application application = null;
		List applications = null;

		try {
			conn = DBUtils.getConnection();

			log.debug("Added a Grants_govTrackingNumber of: " + ggTrackingID);

			application = new Application();
			application.setTRACKING_ID(ggTrackingID);
			applications = new ApplicationDao().searchMatching(conn,
					application);
			if (applications.size() != 1) {
				throw new Exception(
						"No applications matched your request for Grants.gov Tracking ID: "
								+ ggTrackingID);
			}

		} finally {
			try {
				conn.close();
			} catch (Exception ex) {
			}
		}
		return (Application) applications.get(0);
	}
}