package gov.grants.apply.ws;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.OpportunityInformationType;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetOpportunityListRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetOpportunityListResponse;
import gov.grants.apply.system.Global_V1_0.StringMin1Max100Type;
import gov.grants.apply.system.Global_V1_0.StringMin1Max15Type;
import gov.grants.apply.system.Global_V1_0.StringMin1Max255Type;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.GGWSWrapper;
import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.wswrapper.OpportunityListInfo;

/**
 * Processes a GetOpportunityList Web Service Request.
 * 
 * @author Brian Husted
 *  
 */
public class GetOpportunityListProcessor extends AbstractWSProcessor {
	static final Logger log = Logger
			.getLogger(GetOpportunityListProcessor.class.getName());

	/**
	 * @param request
	 *            the Object of the webservice request.
	 * @return the Object of the opportunities.
	 */
	public Object processRequest(Object request) throws Exception {

		//========================================================
		// Declaration of variables
		//========================================================

		_GetOpportunityListRequest getOpportunityListRequest = (_GetOpportunityListRequest) request;
		List opportunities = getOpportunityList(getOpportunityListRequest);
		if (opportunities != null) {
			log.debug("The query returned " + opportunities.size());
		}
		return buildResponse(opportunities);

	}

	protected List getOpportunityList(
			_GetOpportunityListRequest getOpportunityListRequest)
			throws Exception {

		String oppId = hasValue(getOpportunityListRequest.getOpportunityID()) ? getOpportunityListRequest
				.getOpportunityID().getValue()
				: "";

		String cfda = hasValue(getOpportunityListRequest.getCFDANumber()) ? getOpportunityListRequest
				.getCFDANumber().getValue()
				: "";

		String compId = hasValue(getOpportunityListRequest.getCompetitionID()) ? getOpportunityListRequest
				.getCompetitionID().getValue()
				: "";

		return GGWSWrapper.getOpportunityList(cfda, oppId, compId);

	}

	protected _GetOpportunityListResponse buildResponse(List opportunities) {

		_GetOpportunityListResponse response = new _GetOpportunityListResponse();
		List formattedOpportunities = new ArrayList();
		if (opportunities != null) {
			Iterator i = opportunities.iterator();
			int ctr = 0;
			while (i.hasNext()) {
				OpportunityInformationType oppInfoType = new OpportunityInformationType();
				//===============================================
				OpportunityListInfo opp = (OpportunityListInfo) i.next();

				if (opp.getOpportunityNumber() != null) {
					StringMin1Max100Type respOppID = new StringMin1Max100Type(
							opp.getOpportunityNumber());
					oppInfoType.setOpportunityID(respOppID);
				}
				if (opp.getOpportunityTitle() != null) {
					log.debug( "The Opportunity Tile is: " + opp.getOpportunityTitle() );
					StringMin1Max255Type respTitle = new StringMin1Max255Type(
							opp.getOpportunityTitle());
					oppInfoType.setOpportunityTitle(respTitle);
				}
				if (opp.getOpeningDate() != null) {
					oppInfoType.setOpeningDate(opp.getOpeningDate());
				}
				if (opp.getClosingDate() != null) {
					oppInfoType.setClosingDate(opp.getClosingDate());
				}
				StringMin1Max15Type respCFDA = null;
				if (opp.getCfdaNumber() != null) {
					log.debug( "The CFDA is: " + opp.getCfdaNumber());
					respCFDA = new StringMin1Max15Type(opp.getCfdaNumber());
					
				} else {
					//hack for RI remove when releasing new version
					log.debug( "The CFDA is null");
					respCFDA = new StringMin1Max15Type("");
				}
				
				oppInfoType.setCFDANumber(respCFDA);
				
				StringMin1Max100Type respComp = null;
				if (opp.getCompetitionId() != null) {
					log.debug( "The Competition ID is: " + opp.getCompetitionId() );
					respComp = new StringMin1Max100Type(opp.getCompetitionId());
					
				} else {
					//hack for RI remove when releasing new version
					log.debug( "The Competition ID is null");
					respComp = new StringMin1Max100Type("");
					
				}
				
				oppInfoType.setCompetitionID(respComp);

				if (opp.getSchemaURL() != null) {
					log.debug( "The Schema URL is: " + opp.getSchemaURL() );
					oppInfoType.setSchemaURL(opp.getSchemaURL());

				}

				if (opp.getInstructionsURL() != null) {
					log.debug( "The Instruction URL is: " + opp.getInstructionsURL() );
					oppInfoType.setInstructionURL(opp.getInstructionsURL());
				}

				formattedOpportunities.add(oppInfoType);

			}

		}
		if (!formattedOpportunities.isEmpty()) {
			response
					.setOpportunityInformation((OpportunityInformationType[]) formattedOpportunities
							.toArray(new OpportunityInformationType[0]));
		}
		return response;

	}

	/**
	 * @param o
	 *            an object.
	 * @return a true if the object is not null, a false if the object is null.
	 */
	protected static boolean hasValue(Object o) {
		if (o != null) {
			return true;
		}
		return false;

	}

}