/*
 * Created on Jun 24, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package gov.grants.apply.ws;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._SubmitApplicationRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._SubmitApplicationResponse;
import gov.grants.apply.system.Global_V1_0.StringMin1Max240Type;
import gov.grants.apply.util.Utils;

import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.activation.DataSource;

import org.apache.axis.attachments.ManagedMemoryDataSource;
import org.apache.log4j.Logger;

import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.GGWSWrapper;
import com.trw.InFlowSuiteCustomization.Grants_gov.CustomerDefinedClasses.wswrapper.Receipt;

/**
 * Peforms a SubmitApplication call to core Application for further processing.
 * 
 * @author Brian Husted
 *  
 */
public class SubmitApplicationProcessor extends AbstractWSProcessor {
	static final Logger log = Logger.getLogger(SubmitApplicationProcessor.class
			.getName());

	/*
	 * Peforms a SubmitApplication call to core Application. Extracts the
	 * attachements and wrapps them into ByteArrayDataSource objects. The Grant
	 * Application XML is passed as a Java String. Upon a successful call, the
	 * Receipt object is mapped to the Response and passed back to the client
	 * application.
	 * 
	 * @see gov.grants.apply.ws.WSProcessor#processRequest(java.lang.Object)
	 */
	public Object processRequest(Object o) throws Exception {
		Map attachments = null;
		try {
			log.debug("Processing a SubmitApplication Request");
			Calendar receiptDate = Calendar.getInstance();

			attachments = Utils.getSOAPAttachments();
			Receipt receipt = submitApplication((_SubmitApplicationRequest) o,
					(List) attachments.get(Utils.ATTACHMENT_KEY), receiptDate);
			return buildResponse(receipt);
		} finally {
			if (attachments != null) {
				cleanUpCache((List) attachments.get(Utils.DATASOURCE_KEY));
			}
		}

	}

	/**
	 * Apache Axis writes large attachments to the file system for performance.
	 * These files need to be deleted once they are transferred to the back-end.
	 * 
	 * @param attachments
	 */
	private static void cleanUpCache(List attachments) {
		if (attachments == null || attachments.size() <= 0) {
			return;
		}
		Iterator i = attachments.iterator();
		while (i.hasNext()) {

			DataSource dataSource = (DataSource) i.next();
			try {
				if (dataSource instanceof ManagedMemoryDataSource) {
					ManagedMemoryDataSource mmds = (ManagedMemoryDataSource) dataSource;
					if (mmds.delete()) {
						log.debug("Deleted the cached file");
					}
				}
			} catch (Throwable t) {
				log.warn("Unable to delete axis attachment cache file", t);
			}

		}
	}

	private Receipt submitApplication(_SubmitApplicationRequest request,
			List attachments, Calendar receiptDate) throws Exception {

		return GGWSWrapper.submitApplication(getUserId(), (String) request
				.getGrantApplicationXML(), attachments, receiptDate.getTime());

	}

	private _SubmitApplicationResponse buildResponse(Receipt receipt) {
		_SubmitApplicationResponse response = new _SubmitApplicationResponse();
		if (receipt == null) {
			log.debug("The Receipt object is NULL");
		} else {
			log.debug("The Tracking ID is: " + receipt.getTrackingNumber());
		}

		StringMin1Max240Type trkId = new StringMin1Max240Type(receipt
				.getTrackingNumber());
		response.setGrants_govTrackingNumber(trkId);
		Calendar receiptDate = Calendar.getInstance();
		receiptDate.setTime(receipt.getTimestamp());
		response.setReceivedDateTime(receiptDate);
		log.debug("Returning the response for Grants.gov Tracking Number: "
				+ trkId);
		return response;

	}

}