/*
 * Created on Jun 24, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package gov.grants.apply.ws;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._SubmitApplicationRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._SubmitApplicationResponse;
import gov.grants.apply.axis.util.AxisUtils;
import gov.grants.apply.dao.ApplicationDao;
import gov.grants.apply.system.Global_V1_0.StringMin1Max240Type;
import gov.grants.apply.valueObject.Application;
import gov.grants.apply.xpath.GrantApplicationXpath;

import java.io.InputStream;
import java.sql.Connection;
import java.util.Calendar;
import java.util.Random;

import org.apache.axis.attachments.AttachmentPart;
import org.apache.log4j.Logger;

/**
 * Processes a SubmitApplication Web Service Request. Strips key elements from
 * the Grants Gov XML file into the database. Generates a GrantsGov Tracking Id,
 * assigns the application to Received status, and creates a received date and
 * time.
 * 
 * @author Brian Husted
 *  
 */
public class SubmitApplicationProcessor_RI extends AbstractWSProcessor_RI {
	static final Logger log = Logger.getLogger(SubmitApplicationProcessor_RI.class
			.getName());

	private static Random rand = new Random();

	/*
	 * Strips key elements from the Grants Gov XML file into the database.
	 * Generates a GrantsGov Tracking Id, assigns the application to Received
	 * status, and creates a received date and time.
	 * 
	 * @see gov.grants.apply.ws.WSProcessor#processRequest(java.lang.Object)
	 */
	public Object processRequest(Object o) throws Exception {
		log.debug("Processing a SubmitApplication Request");
		AttachmentPart[] attachments = AxisUtils.getMessageAttachments();
		log.debug("Received " + attachments.length + " attachments");
		for (int i = 0; i < attachments.length; i++) {
			log.debug("Attachment Content Type:"
					+ attachments[i].getContentType());

			log
					.debug("Attachment Content-ID: "
							+ attachments[i].getContentId());
			InputStream in = attachments[i].getDataHandler().getInputStream();
			byte[] bytes = new byte[4096];
			int ctr = 0;
			int temp = 0;
			try {
				while ((temp = in.read(bytes, 0, 4096)) > 0) {
					ctr += temp;
				}

				log.debug("The Total size in bytes for attachment with "
						+ "Content-ID: " + attachments[i].getContentId()
						+ " is: " + ctr);

			} catch (Exception ex) {
				log.fatal("Unable to Read Attachment with Content-ID: "
						+ attachments[i].getContentId(), ex);
			} finally {
				try {
					in.close();
				} catch (Exception ex) {
				}
			}

		}
		if (o == null) {
			log.debug("Request object is null");

		}
		log.debug(o.getClass().getName());
		_SubmitApplicationRequest request = (_SubmitApplicationRequest) o;

		Connection conn = null;
		log.debug("Trying to generate a Tracking Id");
		String ggTrackingId = generateGGTrackingId();
		Calendar c = Calendar.getInstance();
		log.debug("Generated a GG Tracking Id");
		try {
			conn = getConnection();
			log.debug("Recieved a database connection");
			GrantApplicationXpath processor = new GrantApplicationXpath(
					(String) request.getGrantApplicationXML());
			
			log.debug("Parsed the Grants.gov xml");
			
			log.debug( "The Hash value is: " + processor.getHeaderHashValue() );
			Application application = new Application();
			log.debug("Attempting to execute XPath queries");
			application.setCFDA_NUMBER(checkValue(processor.getCfdaNumber()));
			log.debug("Executed CFDA XPath");
			application.setCOMPETITION_ID(checkValue(processor
					.getCompetitionId()));
			log.debug("Executed CompID XPath");
			application.setOPPORTUNITY_ID(checkValue(processor
					.getOpportunityId()));
			log.debug("Executed OppID XPath");
			long now = c.getTime().getTime();
			application.setRECEIVED_DATE(now);
			application.setLAST_MODIFY_DATE(now);
			application.setSUBMISSION_TITLE(checkValue(processor
					.getSubmissionTitle()));
			application.setSTATUS("Received");
			application
					.setERROR_LOG("<?xml version='1.0'  standalone='yes'?><Errors> <PROCESSID>"
							+ ggTrackingId
							+ "</PROCESSID> <formname>Error</formname></Errors> ");

			application.setTRACKING_ID(ggTrackingId);
			ApplicationDao dao = new ApplicationDao();
			dao.create(conn, application);
			
			log.debug("Inserted the application in the DB");

		} finally {

			try {
				conn.close();
			} catch (Exception ex) {

			}
		}

		_SubmitApplicationResponse response = new _SubmitApplicationResponse();
		StringMin1Max240Type trkId = new StringMin1Max240Type(ggTrackingId);
		response.setGrants_govTrackingNumber(trkId);
		response.setReceivedDateTime(c);
		log.debug("Returning the response");
		return response;

	}

	protected static String checkValue(String value) {
		return value == null ? "" : value;
	}

	protected String generateGGTrackingId() {
		int random = rand.nextInt(99999);
		String padd = "";
		if (random < 10) {
			padd = "0000";
		} else if (random < 100) {
			padd = "000";
		} else if (random < 1000) {
			padd = "00";
		} else if (random < 10000) {
			padd = "0";
		}

		return "GRANT000" + padd + random;

	}

}