package gov.grants.apply.ws;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationStatusDetailRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetOpportunityListRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._SubmitApplicationRequest;
import gov.grants.apply.util.Utils;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

/**
 * 
 * Concrete factory for instantiating WSProcessors. The key to this class is the
 * processor Map. The Map uses the Web Services Request class name as the key
 * and returns the processor class as the value. No particular instance is
 * associated with this class.
 * 
 * The PRODUCTION_MODE property should be set to false 
 * for Reference implementation users and true for 
 * Grants.gov server deployment.
 * 
 * @author Brian Husted
 *  
 */
public class WSProcessorFactory {

	final static Logger log = Logger.getLogger(WSProcessorFactory.class
			.getName());

	private static Map processorMap = new HashMap();

	static {
		init();
	}

	private WSProcessorFactory() {
		//no instances please...
	}

	private static void init() {
		//		Map of Request objects to Production Processor classes
		if (Utils.isProductionMode()) {
			processorMap.put(_SubmitApplicationRequest.class.getName(),
					SubmitApplicationProcessor.class);
			processorMap.put(_GetOpportunityListRequest.class.getName(),
					GetOpportunityListProcessor.class);
			processorMap.put(_GetApplicationListRequest.class.getName(),
					GetApplicationListProcessor.class);
			processorMap.put(
					_GetApplicationStatusDetailRequest.class.getName(),
					GetApplicationStatusDetailProcessor.class);
		} else{  // The reference implementation processors are mapped here.
			processorMap.put(_SubmitApplicationRequest.class.getName(),
					SubmitApplicationProcessor_RI.class);
			processorMap.put(_GetOpportunityListRequest.class.getName(),
					GetOpportunityListProcessor_RI.class);
			processorMap.put(_GetApplicationListRequest.class.getName(),
					GetApplicationListProcessor_RI.class);
			processorMap.put(
					_GetApplicationStatusDetailRequest.class.getName(),
					GetApplicationStatusDetailProcessor_RI.class);
		}
	}

	/**
	 * Creates the WSProcessor instance based on the request object.
	 * 
	 * @param request
	 *            the Object.
	 * @return
	 * @throws InstantiationException
	 */
	public static WSProcessor createWSProcessor(Object request)
			throws Exception {
		WSProcessor processor = (WSProcessor) ((Class) processorMap.get(request
				.getClass().getName())).newInstance();
		if (processor == null) {
			log.error("The class " + request.getClass().getName()
					+ " could not be associated with a valid WSProcessor");
			return null;
		}

		return processor;
	}

}