/*
 * Created on Jun 24, 2004
 *
 */

package gov.grants.apply.ws;

import java.io.File;

import gov.grants.apply.util.Utils;

import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * A processor for handling Web Services Requests.
 * 
 * @author Brian Husted
 */
public class WSRequestProcessor {
	final static Logger log = Logger.getLogger(WSRequestProcessor.class
			.getName());

	static {
		init();
	}

	/**
	 * Never create an instance
	 */
	private WSRequestProcessor() {

	}

	private static void init() {

		initLog4J();

	}

	private static void initLog4J() {
		try {

			log.debug(":: configuring DOMConfigurator ::");
			String log4Jfile = Utils.getPathToWEB_INF() + "/"
					+ "log4jConfig.xml";
			if (new File(log4Jfile).exists()) {
				DOMConfigurator.configure(log4Jfile);
			} else {
				DOMConfigurator.configure(WSRequestProcessor.class
						.getResource("/resources/" + "log4jConfig.xml"));
				log.debug( "Configured log4J from /resources on classpath (Jar)");
			}
			log.debug(":: configured DOMConfigurator ::");
			log
					.info("** DOMConfigurator :: Logger :: CONFIGURED SUCCESSFULLY **");

		} catch (Throwable t) {
			log.error(t);
		}

	}

	/**
	 * Method uses the WSProcessorFactory to create the appropriate WSProcessor
	 * and exectues the Web Services Request.
	 * 
	 * @param request
	 *            the Object.
	 * @return The Web Service Response Object.
	 * @throws Exception
	 */
	public static Object processWS(Object request) throws Exception {

		WSProcessor processor = WSProcessorFactory.createWSProcessor(request);
		return processor.processRequest(request);

	}

}