/*
 * Created on Jan 6, 2005
 */
package gov.grants.apply.xpath;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Node;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * A class to test the GrantApplicationXpath class.
 * 
 * @author David Wong
 */
public class GrantApplicationXpathTest
    extends TestCase
{
    private static final String HASH_NODE_NAME = "glob:HashValue";
    private static final String HASH_VALUE = "8324afd05d7ac94b398065536f25716e86a24f2a";
    private static final String FORMS_NODE_NAME = "grant:Forms";
    private static final String HEADER_NODE_NAME = "header:GrantSubmissionHeader";

    private GrantApplicationXpath _xpath = null;

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(GrantApplicationXpathTest.class);
    }

    /*
     * @see TestCase#setUp()
     */
    protected void setUp()
        throws Exception
    {
        super.setUp();
        Reader testFile = null;
        StringBuffer stringform = null;
        try
        {
            testFile = new InputStreamReader(new BufferedInputStream(
                GrantApplicationXpathTest.class.getClassLoader()
                    .getResourceAsStream("resources/testFile.xml")));
            stringform = new StringBuffer();
            char[] buffer = new char[1024];
            int charsRead = 0;
            while ((charsRead = testFile.read(buffer, 0, 1024)) > -1)
            {
                stringform.append(buffer, 0, charsRead);
            }
        }
        finally
        {
            try
            {
                testFile.close();
            }
            catch (IOException e)
            {
            }
        }
        _xpath = new GrantApplicationXpath(stringform.toString());
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown()
        throws Exception
    {
        super.tearDown();
    }

    /**
     * Constructor for GrantApplicationXpathTest.
     * 
     * @param methodName
     */
    public GrantApplicationXpathTest(String methodName)
    {
        super(methodName);
    }

    public final void testGetFormsNode()
    {
        Node formsNode = null;
        try
        {
            formsNode = _xpath.getFormsNode();
        }
        catch (TransformerException e)
        {
            Assert.fail(e.getMessage());
        }
        Assert.assertNotNull(
            "The GrantApplicationXpath did not return a Node.", formsNode);
        Assert.assertEquals(
            "The GrantApplicationXpath returned the wrong Node.",
            "grant:Forms", formsNode.getNodeName());
    }

    public final void testGetHeaderNode()
    {
        Node headerNode = null;
        try
        {
            headerNode = _xpath.getHeaderNode();
        }
        catch (TransformerException e)
        {
            Assert.fail(e.getMessage());
        }
        Assert.assertNotNull(
            "The GrantApplicationXpath did not return a Node.", headerNode);
        Assert.assertEquals(
            "The GrantApplicationXpath returned the wrong Node.",
            "header:GrantSubmissionHeader", headerNode.getNodeName());
    }

    public final void testGetHashNode()
    {
        Node hashNode = null;
        try
        {
            hashNode = _xpath.getHashNode();
        }
        catch (TransformerException e)
        {
            Assert.fail(e.getMessage());
        }
        Assert.assertNotNull(
            "The GrantApplicationXpath did not return a Node.", hashNode);
        Assert.assertEquals(
            "The GrantApplicationXpath returned the wrong Node.",
            "glob:HashValue", hashNode.getNodeName());
    }

    public final void testGetHeaderHashValue()
    {
        String hashValue = null;
        try
        {
            hashValue = _xpath.getHeaderHashValue();
        }
        catch (Exception e)
        {
            Assert.fail(e.getMessage());
        }
        Assert
            .assertEquals(
                "The GrantApplicationXpath returned an incorrect value for the hash Node.",
                hashValue, HASH_VALUE);
    }

}